SBT	MACRO		; empty macro
	ENDM

	SBT	arg1 
        SBT     arg2 - one tww
        SBT     arg3 - one two three
        SBT     arg4 - one two three four
        SBT     arg5 - one two three four five
        SBT     arg6 - one two (three)
