        .macro semicolon
        .ascii "; "
        .endm

        .macro colon
        .ascii ": "
        .endm

        semicolon
        .ascii "; "
        colon
        .ascii ": "

	.p2align 5,0
