	fadd.sdd	0f1.0E23,r6,r10	; Immediate form
	fadd.ssd	0f-1.0E23,r6,r10	; Immediate form
	fadd.sss	0f1.0E-23,r6,r10	; Immediate form
	fcmp.sd		0f-1.0E-23,r8,r10	; Immediate form
	fcmp.ss		0f0.0,r8,r10	; Immediate form
	fdiv.sdd	0f2.0,r6,r10	; Immediate form
	fdiv.ssd	0f0.5,r6,r10	; Immediate form
	fdiv.sss	0f4096.0,r6,r10	; Immediate form
	fmpy.sdd	0f-4096.0,r6,r10	; Immediate form
	fmpy.ssd	0f2.0,r6,r10	; Immediate form
	fmpy.sss	0f2.0,r6,r10	; Immediate form
	frndm.sd	0f2.0,r8		; Immediate form
	frndm.si	0f2.0,r8		; Immediate form
	frndm.ss	0f2.0,r8		; Immediate form
	frndm.su	0f2.0,r8		; Immediate form
	frndn.sd	0f2.0,r8		; Immediate form
	frndn.si	0f2.0,r8		; Immediate form
	frndn.ss	0f2.0,r8		; Immediate form
	frndn.su	0f2.0,r8		; Immediate form
	frndp.sd	0f2.0,r8		; Immediate form
	frndp.si	0f2.0,r8		; Immediate form
	frndp.ss	0f2.0,r8		; Immediate form
	frndp.su	0f2.0,r8		; Immediate form
	frndz.sd	0f2.0,r8		; Immediate form
	frndz.si	0f2.0,r8		; Immediate form
	frndz.ss	0f2.0,r8		; Immediate form
	frndz.su	0f2.0,r8		; Immediate form
	fsqrt.sd	0f2.0,r10		; Immediate form
	fsqrt.ss	0f2.0,r10		; Immediate form
	fsub.sdd	0f2.0,r6,r10	; Immediate form
	fsub.ssd	0f2.0,r6,r10	; Immediate form
	fsub.sss	0f2.0,r6,r10	; Immediate form
