MACHINE=
SCRIPT_NAME=elfm68hc12
OUTPUT_FORMAT="elf32-m68hc12"
ROM_START_ADDR=0x08000
ROM_SIZE=0x8000
RAM_START_ADDR=0x01100
RAM_SIZE=0x6F00
TEXT_MEMORY=text
DATA_MEMORY=data
ARCH=m68hc12
MAXPAGESIZE=32
EMBEDDED=yes
GENERIC_BOARD=no
