	.text
	.am33
	add_add r4,r1,r2,r3
	add_add r4,r1,2,r3
	add_sub r4,r1,r2,r3
	add_sub r4,r1,2,r3
	add_cmp r4,r1,r2,r3
	add_cmp r4,r1,2,r3
	add_mov r4,r1,r2,r3
	add_mov r4,r1,2,r3
	add_asr r4,r1,r2,r3
	add_asr r4,r1,2,r3
	add_lsr r4,r1,r2,r3
	add_lsr r4,r1,2,r3
	add_asl r4,r1,r2,r3
	add_asl r4,r1,2,r3
	cmp_add r4,r1,r2,r3
	cmp_add r4,r1,2,r3
	cmp_sub r4,r1,r2,r3
	cmp_sub r4,r1,2,r3
	cmp_mov r4,r1,r2,r3
	cmp_mov r4,r1,2,r3
	cmp_asr r4,r1,r2,r3
	cmp_asr r4,r1,2,r3
	cmp_lsr r4,r1,r2,r3
	cmp_lsr r4,r1,2,r3
	cmp_asl r4,r1,r2,r3
	cmp_asl r4,r1,2,r3
	sub_add r4,r1,r2,r3
	sub_add r4,r1,2,r3
	sub_sub r4,r1,r2,r3
	sub_sub r4,r1,2,r3
	sub_cmp r4,r1,r2,r3
	sub_cmp r4,r1,2,r3
	sub_mov r4,r1,r2,r3
	sub_mov r4,r1,2,r3
	sub_asr r4,r1,r2,r3
	sub_asr r4,r1,2,r3
	sub_lsr r4,r1,r2,r3
	sub_lsr r4,r1,2,r3
	sub_asl r4,r1,r2,r3
	sub_asl r4,r1,2,r3
	mov_add r4,r1,r2,r3
	mov_add r4,r1,2,r3
	mov_sub r4,r1,r2,r3
	mov_sub r4,r1,2,r3
	mov_cmp r4,r1,r2,r3
	mov_cmp r4,r1,2,r3
	mov_mov r4,r1,r2,r3
	mov_mov r4,r1,2,r3
	mov_asr r4,r1,r2,r3
	mov_asr r4,r1,2,r3
	mov_lsr r4,r1,r2,r3
	mov_lsr r4,r1,2,r3
	mov_asl r4,r1,r2,r3
	mov_asl r4,r1,2,r3
	add_add 4,r1,r2,r3
	add_add 4,r1,2,r3
	add_sub 4,r1,r2,r3
	add_sub 4,r1,2,r3
	add_cmp 4,r1,r2,r3
	add_cmp 4,r1,2,r3
	add_mov 4,r1,r2,r3
	add_mov 4,r1,2,r3
	add_asr 4,r1,r2,r3
	add_asr 4,r1,2,r3
	add_lsr 4,r1,r2,r3
	add_lsr 4,r1,2,r3
	add_asl 4,r1,r2,r3
	add_asl 4,r1,2,r3
