.*: Assembler messages:
.*:14: Warning: Use of 'mov' .* WAW dependency 'GR%, % in 1 - 127' \(impliedf\) when entry is at label 'L', specific resource number is 5
.*:13: Warning: This is the location of the conflicting usage
